# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

module Puppet
  Type.newtype(:wls_authentication_provider) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-DESC
    This resource allows you to manage authentication providers in a WebLogic domain.

    Here is an example on how you should use this:

        wls_authentication_provider { 'ldap':
          ensure            => 'present',
          control_flag      => 'SUFFICIENT',
          providerclassname => 'weblogic.security.providers.authentication.LDAPAuthenticator',
          attributes:       =>  'Principal;Host;Port;CacheTTL;CacheSize;MaxGroupMembershipSearchLevel;SSLEnabled',
          attributesvalues  =>  'ldapuser;ldapserver;389;60;1024;4;1',
          order             =>  '0'
        }

    To manage Weblogic's DefaultIdentityAsserter use the wls_identity_asserter type. In this example
    you are managing a provider in the default domain. When you want to
    manage a provider in a specific domain, you can use:

        wls_authentication_provider { 'MyDomain/MyAuthenticationProvider':
          ensure           => 'present',
        }

    Check [the Oracle Weblogic documentation](http://docs.oracle.com/middleware/1221/wls/INTRO/clustering.htm#INTRO185)
    for more documentation about setting up and managing a WebLogic cluster.

    DESC

    ensurable

    to_get_raw_resources do
      Puppet.info "index #{name} "
      environment = { 'action' => 'index', 'type' => 'wls_authentication_provider' }
      wlst template('wls_config/wls_authentication_provider/index.py.erb', binding), environment
    end

    on_create do
      Puppet.info "create #{name} "
      wlst template('wls_config/wls_authentication_provider/create.py.erb', binding)
      nil
    end

    on_modify do
      Puppet.info "modify #{name} "
      wlst template('wls_config/wls_authentication_provider/modify.py.erb', binding)
      nil
    end

    on_destroy do
      Puppet.info "destroy #{name} "
      wlst template('wls_config/wls_authentication_provider/destroy.py.erb', binding)
      nil
    end

    parameter :authentication_provider_name
    parameter :domain
    parameter :name
    parameter :timeout

    property :control_flag
    parameter :providerclassname
    property  :provider_specific
    parameter :attributes
    parameter :attributesvalues
    parameter :order

    add_title_attributes(:authentication_provider_name)
  end
end
