# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

# Doc
module Puppet
  Type.newtype(:wls_jndi_security_policy) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-DESC
    This resource allows you to manage jndi security authorization policies in an WebLogic Security Realm.

    Here is an example on how you should use this:

        wls_jndi_security_policy { 'domain/managed_server:javax.jms.TopicConnectionFactory:modify':
          ensure           => 'present',
          policyexpression => 'Role(Admin) | Role(Deployer)',
        }

    This puppet snippit set's the security policy for `modify` operation of the the JNDI object
    `javax.jms.TopicConnectionFactory` on server `managed_server` in the domain `domain`
    to: `Role(Admin) | Role(Deployer)`.

    DESC

    ensurable

    to_get_raw_resources do
      Puppet.info "index #{name}"
      environment = { 'action' => 'index', 'type' => 'wls_jndi_security_policy' }
      wlst template('wls_config/wls_jndi_security_policy/index.py.erb', binding), environment
    end

    on_create do
      wlst_action = 'create'
      Puppet.info "create #{name}"
      wlst template('wls_config/wls_jndi_security_policy/create.py.erb', binding)
      nil
    end

    on_modify do
      wlst_action = 'modify'
      Puppet.info "modify #{name}"
      wlst template('wls_config/wls_jndi_security_policy/modify.py.erb', binding)
      nil
    end

    on_destroy do
      Puppet.info "destroy #{name}"
      wlst template('wls_config/wls_jndi_security_policy/destroy.py.erb', binding)
      nil
    end

    parameter :domain
    parameter :name
    parameter :server_name
    parameter :jndi_name
    parameter :action
    parameter :authorizationprovider
    parameter :timeout
    property  :policyexpression

    add_title_attributes(:server_name, :jndi_name, :action) do
      /(.+):(.+):(.+)/
    end

    def internal_path
      "{#{jndi_name.tr('.', ',')}}"
    end

    def encoded_path
      jndi_name.split('.').join('@M')
    end
  end
end
