# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.dirname(__FILE__)
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../../../easy_type/lib')
module Puppet
  Type.newtype(:wls_managedserver) do
    def self.module_name
      'wls_config'
    end
    desc <<-DESC
    control a managed server or cluster state like running,stop,restart

    Type for managed server control like start, running, abort and stop a managed server or a cluster.
    Also supports `subscribe` with `refreshonly` when you only want to execute the restart when something
    has changed.

    Here is an example on how you should use this:

        wls_managedserver{'JMSServer1_Wls1036:':
          ensure                    => 'running',   #running|start|abort|stop
          target                    => 'Server', #Server|Cluster
          server_name               => 'JMSServer1',
          domain_name               => hiera('domain_name'),
          os_user                   => hiera('wls_os_user'),
          weblogic_home_dir         => hiera('wls_weblogic_home_dir'),
          weblogic_user             => hiera('wls_weblogic_user'),
          weblogic_password         => hiera('domain_wls_password'),
          jdk_home_dir              => hiera('wls_jdk_home_dir'),
          adminserver_address       => hiera('domain_adminserver_address'),
          adminserver_port          => hiera('domain_adminserver_port'),
        }

    Here is an example with `refreshonly`. In this example, the restart will only be executed
    when the domain `Wls1036`  is changed. A restart will be executed.

        wls_managedserver{'JMSServer1_Wls1036':
          ensure                    => 'restart',   #running|start|abort|stop
          target                    => 'Server',    #Server|Cluster
          server_name               => 'JMSServer1',
          domain_name               => hiera('domain_name'),
          os_user                   => hiera('wls_os_user'),
          weblogic_home_dir         => hiera('wls_weblogic_home_dir'),
          weblogic_user             => hiera('wls_weblogic_user'),
          weblogic_password         => hiera('domain_wls_password'),
          jdk_home_dir              => hiera('wls_jdk_home_dir'),
          adminserver_address       => hiera('domain_adminserver_address'),
          adminserver_port          => hiera('domain_adminserver_port'),
          refreshonly               => true,
          subscribe                 => Wls_domain['Wls1036'],
        }

    DESC

    newproperty(:ensure) do
      include EasyType
      desc <<-DESC
      Whether to do something.
      DESC

      newvalue(:start, :event => :managedserver_running) do
        provider.start unless :refreshonly == true
      end

      newvalue(:stop, :event => :managedserver_running) do
        provider.stop unless :refreshonly == true
      end

      aliasvalue(:running, :start)
      aliasvalue(:abort, :stop)
      data_type("Optional[Enum['running', 'start', 'abort', 'stop']]")
      def retrieve
        provider.status
      end

      def sync
        event = super()
        # rubocop:disable all
        if property = @resource.property(:enable)
          val = property.retrieve
          property.sync unless property.safe_insync?(val)
        end
        # rubocop:enable all
        event
      end
    end

    newparam(:name) do
      include EasyType
      desc <<-DESC
        The title.
      DESC
      data_type('Optional[String]')
      isnamevar
    end

    newparam(:target) do
      include EasyType
      desc <<-DESC
        The weblogic type.
      DESC
      data_type('Optional[String]')
    end

    newparam(:server_name) do
      include EasyType
      desc <<-DESC
        The adminserver name.
      DESC
      data_type('Optional[String]')
    end

    newparam(:domain_name) do
      include EasyType
      desc <<-DESC
        The weblogic domain name.
      DESC
      data_type('Optional[String]')
    end

    newparam(:os_user) do
      include EasyType
      desc <<-DESC
        The weblogic operating system user.
      DESC
      data_type('Optional[String]')
    end

    newparam(:weblogic_home_dir) do
      include EasyType
      desc <<-DESC
        The weblogic home folder.
      DESC
      data_type('Optional[String]')
    end

    newparam(:weblogic_user) do
      include EasyType
      desc <<-DESC
        The weblogic user.
      DESC
      data_type('Optional[String]')
    end

    newparam(:weblogic_password) do
      include EasyType
      desc <<-DESC
        The weblogic user password.
      DESC
      data_type('Optional[String]')
    end

    newparam(:jdk_home_dir) do
      include EasyType
      desc <<-DESC
        The jdk home dir.
      DESC
      data_type('Optional[String]')
    end

    newparam(:adminserver_address) do
      include EasyType
      desc <<-DESC
        The adminserver address.
      DESC
      data_type('Optional[String]')
    end

    newparam(:adminserver_port) do
      include EasyType
      desc <<-DESC
        The adminserver port.
      DESC
      data_type("Optional[Variant[Enum['absent'],Integer]]")
    end

    newparam(:refreshonly) do
      include EasyType
      desc <<-DESC
        The command should only be run as a
        refresh mechanism for when a dependent object is changed.
      DESC
      # rubocop:disable Lint/BooleanSymbol
      newvalues(:true, :false)
      data_type('Optional[Boolean]')
      # rubocop:enable Lint/BooleanSymbol
      defaultto false
    end

    def refresh
      Puppet.info 'wls_managedserver refresh'
      provider.restart
    end
  end
end
