# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

module Puppet
  Type.newtype(:wls_multi_datasource) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-DESC
    This resource allows you to manage a multi datasource in an WebLogic domain.

    Here is an example on how you should use this:

        wls_multi_datasource { 'myMultiDatasource':
          ensure        => 'present',
          algorithmtype => 'Failover',
          datasources   => ['myJDBCDatasource'],
          jndinames     => ['myMultiDatasource'],
          target         => ['ManagedServer1', 'WebCluster'],
          targettype     => ['Server', 'Cluster'],
          testfrequency => '120',
        }

    In this example you are managing a multi datasource in the default domain. When you want to
    manage a multi datasource in a specific domain, you can use:

        wls_multi_datasource { 'my_domain/myMultiDatasource':
          ensure        => 'present',
          algorithmtype => 'Failover',
          datasources   => ['myJDBCDatasource'],
          jndinames     => ['myMultiDatasource'],
          target         => ['ManagedServer1', 'WebCluster'],
          targettype     => ['Server', 'Cluster'],
          testfrequency => '120',
        }

    DESC

    index_condition do |name, path, wls_config|
      datasource_list = wls_config.get("#{path}/#{name}/JDBCResource/#{name}/JDBCDataSourceParams/#{name}/DataSourceList")
      !datasource_list.nil?
    end

    on_create do
      Puppet.info "create #{name} "
      create_resource
      #
      # JDBC Datasource requires that you also set the name attribute
      #
      key = "/JDBCSystemResources/#{multi_datasource_name}/JDBCResource/#{multi_datasource_name}/Name"
      wls_config.set(key, multi_datasource_name)
    end

    bean_type('JDBCSystemResource')

    ensurable
    parameter :domain
    parameter :name
    parameter :multi_datasource_name

    property :target
    property :targettype
    property :jndinames
    property :datasources
    property :algorithmtype
    property :testfrequency
    extraproperties do
      property :ds_connection_pool_failover_callback_handler
      property :ds_scope
      property :xa_end_only_once
      property :xa_keep_logical_conn_open_on_release
      property :xa_keep_xa_conn_till_tx_complete
      property :xa_need_tx_ctx_on_close
      property :xa_new_xa_conn_for_commit
      property :xa_recover_only_once
      property :xa_resource_health_monitoring
      property :xa_retry_duration_seconds
      property :xa_retry_interval_seconds
      property :xa_rollback_local_tx_upon_conn_close
      property :xa_set_transaction_timeout
      property :xa_transaction_timeout
      property :ds_algorithm_type
      property :ds_data_source_list
      property :ds_failover_request_if_busy
      property :ds_global_transactions_protocol
      property :ds_jndi_names
      property :ds_keep_conn_after_global_tx
      property :ds_keep_conn_after_local_tx
      property :ds_row_prefetch
      property :ds_row_prefetch_size
      property :ds_stream_chunk_size
      property :oracle_active_gridlink
      property :oracle_affinity_policy
      property :driver_name
      property :driver_password
      property :driver_url
      property :driver_use_password_indirection
      property :driver_use_xa_data_source_interface
      property :oracle_connection_initialization_callback
      property :oracle_enable_java_net_fast_path
      property :oracle_fan_enabled
      property :oracle_ons_node_list
      property :oracle_ons_wallet_file
      property :oracle_ons_wallet_password
      property :oracle_optimize_utf8_conversion
      property :oracle_proxy_session
      property :oracle_replay_initiation_timeout
      property :oracle_use_database_credentials
      property :pool_capacity_increment
      property :pool_connection_creation_retry_frequency_seconds
      property :pool_connection_labeling_callback
      property :pool_count_of_refresh_failures_till_disable
      property :pool_identity_based_connection_pooling_enabled
      property :pool_initial_capacity
      property :pool_init_sql
      property :pool_profile_harvest_frequency_seconds
      property :pool_statement_cache_type
      property :pool_connection_harvest_max_count
      property :pool_connection_reserve_timeout_seconds
      property :pool_count_of_test_failures_till_flush
      property :pool_credential_mapping_enabled
      property :pool_driver_interceptor
      property :pool_fatal_error_codes
      property :pool_highest_num_waiters
      property :pool_ignore_in_use_connections_enabled
      property :pool_inactive_connection_timeout_seconds
      property :pool_jdbcxa_debug_level
      property :pool_login_delay_seconds
      property :pool_max_capacity
      property :pool_min_capacity
      property :pool_pinned_to_thread
      property :pool_seconds_to_trust_an_idle_pool_connection
      property :pool_shrink_frequency_seconds
      property :pool_statement_timeout
      property :pool_test_frequency_seconds
      property :pool_test_table_name
      property :pool_wrap_jdbc
      property :deployment_order
      property :pool_connection_harvest_trigger_count
      property :pool_profile_type
      property :pool_remove_infected_connections
      property :pool_statement_cache_size
      property :pool_test_connections_on_reserve
      property :pool_wrap_types
      property :compatibility_name
      property :deployment_principal_name
      property :module_type
      property :notes
      property :source_path
      #
      # Weblogic 12.2.1 new properties
      #
      property :ds_proxy_switching_callback
      property :ds_proxy_switching_properties
      property :pool_profile_connection_leak_timeout_seconds
      property :tags
    end

    add_title_attributes(:multi_datasource_name)
    child_of(:wls_datasource) { datasources }
  end
end
